"""
Copyright (c) 2023 Henry Schreiner. All rights reserved.

check-sdist: Check the contents of an SDist vs. git
"""

from __future__ import annotations

__version__ = "1.4.0"

__all__ = ["__version__"]
